package com.extensis.web.common.client;

import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.logging.Logger;

import com.extensis.web.common.WebFontJMSException;
import com.extensis.web.common.JMS.TopicListenerIF;
import com.extensis.web.common.JMS.TopicListenerStandalone;
import com.extensis.web.common.JMS.TopicPublisher;
import com.extensis.web.common.JMS.Topics;

import org.apache.commons.cli.*;


public class jmster {
	private static Options options = null; // Command line options
	
	private static final String CMD_LINE = "java -jar jmster.jar";
	
	private static final String HOST_OPTION = "host";
	private static final String FILE_OPTION = "file";
	private static final String MODE_OPTION = "mode";
	private static final String TOPIC_OPTION = "topic";
	
	private static final String MODE_PUBLISH = "publish";
	private static final String MODE_SUBSCRIBE = "subscribe";
	
	static{
		options = new Options();
		options.addOption(OptionBuilder.withArgName(MODE_PUBLISH + "|" + MODE_SUBSCRIBE)
                .hasArg()
                .isRequired()
                .withDescription("Declare whether jmster will run as a topic publisher or subscriber")
                .create(MODE_OPTION));
		options.addOption(OptionBuilder.withArgName("hostName")
                .hasArg()
                .isRequired()
                .withDescription("JMS server name or IP address")
                .create(HOST_OPTION));
		options.addOption(OptionBuilder.withArgName("topicName")
                .hasArg()
                .isRequired()
                .withDescription("JMS topic name")
                .create(TOPIC_OPTION));
		options.addOption(OptionBuilder.withArgName("filename")
                .hasArg()
                .withDescription("File name of the file defining the message contents. NOTE: Must be supplied when mode=publish.")
                .create(FILE_OPTION));
	}

	private static String file = null;
	private static String host = null;
	private static String mode = null;
	private static String topic = null;

	private static Map<String, Object> messageMap = null;
	
	public static void main(String[] args) {
		loadArgs(args);

		if (mode.equals(MODE_PUBLISH)) {
			runAsPublisher();
		}
		else {
			runAsSubscriber();
		}
	}

	/**
	 * Validate and set command line arguments.
	 * Exit after printing usage if anything is astray
	 * @param args String[] args as featured in public static void main()
	 */
	private static void loadArgs(String[] args){
		CommandLineParser parser = new GnuParser();

		CommandLine cmd = null; // Command Line arguments
		try {
			cmd = parser.parse(options, args);
		} catch (MissingArgumentException mae) {
			System.err.println(mae.getMessage());
			HelpFormatter formatter = new HelpFormatter();
			formatter.printHelp(CMD_LINE, options, true);
			System.exit(1);
		} catch (MissingOptionException moe) {
			System.err.println("Required parameter not found: " + moe.getMessage());
			HelpFormatter formatter = new HelpFormatter();
			formatter.printHelp(CMD_LINE, options, true);
			System.exit(1);
		} catch (ParseException pe) {
			System.err.println(pe);
			HelpFormatter formatter = new HelpFormatter();
			formatter.printHelp(CMD_LINE, options, true);
			System.exit(1);
		}
		
		file = cmd.getOptionValue(FILE_OPTION);
		host = cmd.getOptionValue(HOST_OPTION);
		mode = cmd.getOptionValue(MODE_OPTION);
		topic = cmd.getOptionValue(TOPIC_OPTION);
		
		if (mode.equals(MODE_PUBLISH)) {
			if (file == null) {
				System.err.println("Must supply a file when mode=publish.");
				HelpFormatter formatter = new HelpFormatter();
				formatter.printHelp(CMD_LINE, options, true);
				System.exit(1);
			}
			if (!new File(file).exists()) {
				System.err.println("File " + file + " does not exist.");
				System.exit(1);
			}
			if (!loadMessageMap()) {
				System.err.println("Unable to parse file " + file + ".");
				System.exit(1);
			}
		}
		else if (mode.equals(MODE_SUBSCRIBE)) {
		}
		else {
			System.err.println("Invalid mode parameter.");
			HelpFormatter formatter = new HelpFormatter();
			formatter.printHelp(CMD_LINE, options, true);
			System.exit(1);
		}
	}

	private static boolean loadMessageMap() {
		boolean rc = true;
		
		try {
	        Properties props = new Properties();
	        FileInputStream fis = new FileInputStream(file);
	        props.load(fis);    
	        fis.close();
	        
	        messageMap = new HashMap<String, Object>();
	        
	        for (Enumeration e=props.propertyNames(); e.hasMoreElements();) {
	        	String propertyName = (String)e.nextElement();
	        	
	        	String propertyValue = props.getProperty(propertyName);
	        	
	        	messageMap.put(propertyName, propertyValue);
	        }
		}
		catch (IOException e) {
			rc = false;
		}
		
		return rc;
	}
	
	private static void runAsPublisher() {
		// Load message file into a Map
		try {
			TopicPublisher.sendMessage(topic, Topics.SERVLET_CONNECTION_FACTORY_NAME, messageMap);
		}
		catch (WebFontJMSException e) {
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	private static void runAsSubscriber() {
		try {
			// Start topic listener
			TopicListenerStandalone myTopicListener = new TopicListenerStandalone(topic, host, new MyTopicListener());
			
			// wait for keystroke
			try {
				System.out.println("Listening to topic");
				System.in.read();
			}
			catch (IOException e) {
				e.printStackTrace();
			}
			
			// Stop topic listener
			myTopicListener.close();
		}
		catch (WebFontJMSException e) {
			e.printStackTrace();
			System.exit(1);
		}
	}
	
	private static class MyTopicListener implements TopicListenerIF {
		public void onMessage(Map<String, Object> msgContents) {
			System.out.println("<message>");
			for (String key : msgContents.keySet()) {
				System.out.println("    <" + key + ">" + msgContents.get(key).toString() + "</" + key + ">");
			}
			System.out.println("</message>");
		}
	}
}
