package com.extensis.web.common.JMS;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.extensis.web.common.WebFontJMSException;

public class TopicListener implements MessageListener {
	private TopicListenerIF topicListener = null;
	private TopicConnection topicConnection = null;

	public TopicListener(String topicName, String connectionFactoryName, TopicListenerIF topicListener) throws WebFontJMSException {
		if (null==topicName || topicName.length()==0) {
			throw new IllegalArgumentException("topicName cannot be null or empty");
		}
		if (null==connectionFactoryName || connectionFactoryName.length()==0) {
			throw new IllegalArgumentException("connectionFactoryName cannot be null or empty");
		}
		if (null==topicListener) {
			throw new IllegalArgumentException("topicListener cannot be null");
		}

		boolean success = false;

		this.topicListener = topicListener;

		try {
			Context initialContext = new InitialContext();
	
			try {
				String topicJNDIName = Topics.TOPIC_JNDI_PREFIX + topicName;
				Topic topic = (Topic) initialContext.lookup(topicJNDIName);
				
				TopicConnectionFactory topicConnectionFactory = (TopicConnectionFactory) initialContext.lookup(connectionFactoryName);
	
				topicConnection = topicConnectionFactory.createTopicConnection();
	
				try {
					TopicSession topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
	
					TopicSubscriber topicSubscriber = topicSession.createSubscriber(topic);
	
					topicSubscriber.setMessageListener(this);
	
					topicConnection.start();
					
					success = true;
				} finally {
					if (!success) {
						topicConnection.close();
					}
				}
			}
			finally {
				initialContext.close();
			}
		}
		catch (NamingException ne) {
			throw new WebFontJMSException(ne);
		}
		catch (JMSException jmse) {
			throw new WebFontJMSException(jmse);
		}
	}

	@SuppressWarnings(value = "unchecked")
	public void onMessage(Message msg) {
		try {
			// We know this is a MapMessage, so let's cast it here
			MapMessage mapMsg = (MapMessage) msg;

			Map<String, Object> msgContents = new HashMap<String, Object>();
			for (Enumeration<String> e=(Enumeration<String>)mapMsg.getMapNames(); e.hasMoreElements();) {
				String mapName = e.nextElement();
				msgContents.put(mapName, mapMsg.getObject(mapName));
			}
			topicListener.onMessage(msgContents);
		}
		catch (Throwable t) {
			// This should never throw anything, so we'll eat all exceptions here
			t.printStackTrace();
		}
	}

	public void close() throws WebFontJMSException {
		try {
			topicConnection.close();
		}
		catch (JMSException jmse) {
			throw new WebFontJMSException(jmse);
		}
	}
}
