package com.extensis.web.common.JMS;

import org.hornetq.api.core.TransportConfiguration;
import org.hornetq.integration.transports.netty.NettyConnectorFactory;
import org.hornetq.integration.transports.netty.TransportConstants;
import org.hornetq.jms.client.HornetQTopic;
import org.hornetq.jms.client.HornetQConnectionFactory;

import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;

import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.Session;
import javax.jms.Topic;
import javax.jms.TopicConnection;
import javax.jms.TopicConnectionFactory;
import javax.jms.TopicSession;
import javax.jms.TopicSubscriber;

import com.extensis.web.common.WebFontJMSException;

public class TopicListenerStandalone implements MessageListener {
	private static TopicConnectionFactory topicConnectionFactory = null;

	private TopicListenerIF topicListener = null;
	private TopicConnection topicConnection = null;
	private TopicSession topicSession = null;

	public TopicListenerStandalone(String topicName, String jmsHostName, TopicListenerIF topicListener) throws WebFontJMSException {
		if (null==topicName || topicName.length()==0) {
			throw new IllegalArgumentException("topicName cannot be null or empty");
		}
		if (null==jmsHostName || jmsHostName.length()==0) {
			throw new IllegalArgumentException("jmsHostName cannot be null or empty");
		}
		if (null==topicListener) {
			throw new IllegalArgumentException("topicListener cannot be null");
		}

		Map<String, Object> connectionParams = new HashMap<String, Object>();
		connectionParams.put(TransportConstants.HOST_PROP_NAME, jmsHostName);
		connectionParams.put(TransportConstants.PORT_PROP_NAME, 5445);
		TransportConfiguration transportConfiguration = new TransportConfiguration(NettyConnectorFactory.class.getName(), connectionParams);
		topicConnectionFactory = (TopicConnectionFactory) new HornetQConnectionFactory(transportConfiguration);
		if (null == topicConnectionFactory) {
			throw new WebFontJMSException("HornetQ initialization failed");
		}

		boolean success = false;

		this.topicListener = topicListener;

		try {
	         // Directly instantiate the JMS Topic object.
			Topic topic = new HornetQTopic(topicName);

			// Create a JMS TopicConnection
			topicConnection = topicConnectionFactory.createTopicConnection();

			try {
				// Create a JMS TopicSession
				topicSession = topicConnection.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);

				// Create a JMS topic subscriber with this instance as the listener
				TopicSubscriber topicSubscriber = topicSession.createSubscriber(topic);
				topicSubscriber.setMessageListener(this);

				// Start the Connection
				topicConnection.start();

				success = true;
			} finally {
				if (!success) {
					topicConnection.close();
					topicConnection = null;
				}
			}
		}
		catch (JMSException jmse) {
			throw new WebFontJMSException(jmse);
		}
	}

	@SuppressWarnings(value = "unchecked")
	public void onMessage(Message msg) {
		try {
			// We know this is a MapMessage, so let's cast it here
			MapMessage mapMsg = (MapMessage) msg;

			Map<String, Object> msgContents = new HashMap<String, Object>();
			for (Enumeration<String> e=(Enumeration<String>)mapMsg.getMapNames(); e.hasMoreElements();) {
				String mapName = e.nextElement();
				msgContents.put(mapName, mapMsg.getObject(mapName));
			}
			topicListener.onMessage(msgContents);
		}
		catch (Throwable t) {
			// This should never throw anything, so we'll eat all exceptions here
			t.printStackTrace();
		}
	}

	public void close() throws WebFontJMSException {
		try {
			topicConnection.close();
		}
		catch (JMSException jmse) {
			throw new WebFontJMSException(jmse);
		}
	}
}
